%% Nonlinar Regression Model

close all
clear all
clc

%% Specify model parameters. The Hill function is given by y(x) = ymax/(1 + (EC50/dose)^n)
ymax = 1;
Lec50 = 0;
n = 2;
modelpar = [ymax Lec50 n];

%% Generate data
dose = (logspace(-1, 1, 15))';
yTrue = Hill(modelpar, dose);

%% Add normal error
NoiseStd = 0.1;
err = NoiseStd*randn(length(yTrue), 1);
yExpt = yTrue + err;

%% NonLinear Regression: nlinfit
options = statset('Display', 'iter');
betaGuess = [1, 1, 1]
betaHat = nlinfit(dose, yExpt, @Hill, betaGuess, options)

% Change initial value
betaGuess_c = [0.1, 1, 4]
betaHat_c = nlinfit(dose, yExpt, @Hill, betaGuess, options)
 doseFit = (logspace(-1, 1, 201))';
yfit_c = Hill(betaHat_c, doseFit);
%% Plot data
 figure(1)
 semilogx(dose, yExpt, 'ro', 'MarkerSize', 8)
 hold on
 doseFit = (logspace(-1, 1, 201))';
 yFit = Hill(betaHat, doseFit);
 plot(doseFit, yFit, 'b-', 'LineWidth', 2)
 plot(doseFit, Hill(modelpar, doseFit), 'b--', 'LineWidth', 2)
 xlim([0.09 11])
 ylim([-0.05 1.15])
 xlabel('Dose')
 ylabel('Response')
 grid on
 legend('Data', 'Nonlinear fit', 'True model', 'Location', 'NW')
 set(gcf,'color','white');
 
%% Plot Residuals
yPredicted = interp1(doseFit, yFit, dose);
residuals = yExpt - yPredicted;

figure()
semilogx(dose, residuals, 'bo--', 'MarkerSize', 7, 'MarkerFaceColor', 'b')
xlabel('Dose')
ylabel('Residuals')
grid on
legend('Residuals', 'Location', 'NW')
set(gcf,'color','white');

%% Polynomial regression. Generate a some data for a saturable curve model. Add normally distributed noise. Fit with a 2nd order polynomial
ymax = 1;
tau = 3;

%% Generate data
t = (1:10)';
yTrue = ymax*(1-exp(-t/tau));

%% Add normal error 
NoiseStd = 0.1; 
err = NoiseStd*randn(length(t), 1);
yExpt = yTrue + err;

%% Fit with a 2nd order polynomial. Create design matrix
A = [ones(size(t)) t t.^2];

%% Solve normal equation using the MATLAB backslash operator
betaHat = A \ yExpt

%% Plot data and fit
figure(3)
plot(t, yExpt, 'ro', 'MarkerSize', 8)
hold on
tFit = (1:0.1:10)';
yFit = [ones(size(tFit)) tFit tFit.^2]*betaHat;
plot(tFit, yFit, 'b-', 'LineWidth', 2)
plot(tFit, ymax*(1-exp(-tFit/tau)), 'b--', 'LineWidth', 1)
xlim([0, 11])
ylim([0, 1.1])
xlabel('x')
ylabel('y')
grid on
legend('Data', 'Polynomial fit', 'True function', 'Location', 'NW')
set(gcf,'color','white');
%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Estimate asymptotic confidence intervals for a nonlinear regression
% fit using nlparci (Statistics toolbox).
% Fit simulated dose-response data to a Hill function, given by 
% y(x) = ymax/(1 + (EC50/dose)^n)
%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Load data
load('DoseData1.mat');
dose = DoseData1(:,1);
yExpt = DoseData1(:,2);

%% Call nlinfit to perform nonlinear regression.
betaGuess = [1, 1, 1]; % Initial guess for parameter values
[betaHat, res, Jac, Cov, mse] = nlinfit(dose, yExpt, @Hill, betaGuess);
%Supply output to nlparci to estimate asymptotic 95% CIs. 
asympCI = nlparci(betaHat, res, 'covar', Cov);
% Print parameter estimates and asymptotic CIs
[betaHat' asympCI]


